@@ -13,14 +13,12 @@ import threading
 
 gi.require_version('Adw', '1')
 gi.require_version('Gtk', '4.0')
-gi.require_version('Notify', '0.7')
 
 from gi.repository import Adw
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
-from gi.repository import Notify
 
 from komikku.card import CardPage
 from komikku.consts import CREDITS
@@ -115,7 +113,6 @@ class Application(Adw.Application):
 
         init_db()
         init_servers_modules(Settings.get_default().external_servers_modules)
-        Notify.init('Komikku')
 
 
 @Gtk.Template.from_resource('/info/febvre/Komikku/ui/application_window.ui')
@@ -465,7 +462,8 @@ available in your region/language."""))
         def on_maximized():
             # Gtk.Window::maximized (idem with Gdk.Toplevel:state) event is unreliable because it's emitted too earlier
             # We detect that maximization is effective by comparing monitor size and window size
-            monitor_width = self.monitor.props.geometry.width / self.get_scale_factor()
+            scale_factor = self.get_native().get_surface().get_scale()
+            monitor_width = self.monitor.props.geometry.width / scale_factor
             if self.get_width() < monitor_width and self.is_maximized():
                 return True
 
@@ -583,7 +581,7 @@ available in your region/language."""))
             self.library.select_all()
         elif self.page == 'card':
             self.card.chapters_list.select_all()
-        elif self.page == 'download_manager':
+        elif self.page == 'download-manager':
             self.download_manager.select_all()
 
     def show_banner(self, title, at_bottom=False):
