@@ -361,6 +361,9 @@ class ChaptersList:
             self.update_chapter_item(chapter=chapter)
 
     def select_all(self, *args):
+        if self.card.stack.get_visible_child_name() != 'chapters':
+            return
+
         if not self.card.selection_mode:
             self.card.enter_selection_mode()
 
@@ -612,7 +615,7 @@ class ChaptersListRow(Gtk.Box):
         if not update:
             item.connect('changed', self.populate, True)
             self.download_stop_button_clicked_handler_id = self.download_stop_button.connect(
-                'clicked', lambda _button, chapter: self.card.window.downloader.remove(chapter), self.chapter
+                'clicked', lambda _button, chapter: self.card.window.downloader.cancel([chapter]), self.chapter
             )
 
         self.title_label.set_label(self.chapter.title)
