@@ -21,6 +21,7 @@ from komikku.consts import MISSING_IMG_RESOURCE_PATH
 from komikku.utils import convert_and_resize_image
 from komikku.utils import CoverPicture
 from komikku.utils import html_escape
+from komikku.utils import log_error_traceback
 
 THUMB_WIDTH = 96
 THUMB_HEIGHT = 136
@@ -149,7 +150,8 @@ class TracherResultRow(Gtk.ListBoxRow):
         if data.get('start_date'):
             details.append(data['start_date'])
 
-        details.append(data['status'])
+        if data.get('status'):
+            details.append(data['status'])
 
         if data.get('score'):
             details.append(f'{data["score"]}/10')
@@ -247,11 +249,11 @@ class TrackerRow(Adw.ExpanderRow):
             manga.update({'tracking': manga.tracking})
             self.init()
 
-        self.window.confirm(
+        self.window.open_dialog(
             _('Delete?'),
-            _('Are you sure you want to cancel tracking?'),
-            _('Delete'),
-            confirm_callback,
+            body=_('Are you sure you want to cancel tracking?'),
+            confirm_label=_('Delete'),
+            confirm_callback=confirm_callback,
             confirm_appearance=Adw.ResponseAppearance.DESTRUCTIVE
         )
 
@@ -279,7 +281,8 @@ class TrackerRow(Adw.ExpanderRow):
             try:
                 id = self.window.card.manga.tracking[self.tracker.id]['id']
                 data = self.tracker.get_manga_data(id)
-            except Exception:
+            except Exception as e:
+                log_error_traceback(e)
                 data = None
 
             if data is None:
@@ -295,8 +298,15 @@ class TrackerRow(Adw.ExpanderRow):
             self.set_enable_expansion(True)
             self.set_expanded(True)
             self.set_arrow_visible(True)
+            self.set_activatable(True)
+            self.set_subtitle('')
             self.btn.set_visible(False)
-            self.action_row.set_title(f'<a href="{self.tracker.get_manga_url(data['id'])}">{html_escape(data["name"])}</a>')
+
+            if data.get('url'):
+                tracker_manga_url = data['url']
+            else:
+                tracker_manga_url = self.tracker.get_manga_url(data['id'])
+            self.action_row.set_title(f'<a href="{tracker_manga_url}">{html_escape(data["name"])}</a>')
 
             with self.chapters_progress_row.handler_block(self.num_chapter_changed_handler_id):
                 adj = Gtk.Adjustment(
@@ -321,33 +331,36 @@ class TrackerRow(Adw.ExpanderRow):
             with self.status_row.handler_block(self.status_changed_handler_id):
                 self.status_row.set_selected(self.tracker.get_status_index(data['status']))
 
-        def reset():
-            self.set_expanded(False)
-            self.set_enable_expansion(False)
-            self.set_arrow_visible(False)
-            self.btn.set_visible(True)
-            self.action_row.set_title('')
-
-        # Is the tracker connected and active?
-        tracker_data = self.tracker.get_data()
-        active = tracker_data and tracker_data['active']
-        self.set_visible(active)
+        # Is the tracker granted?
+        granted, access_token_valid = self.tracker.is_granted()
+        self.set_visible(granted)
 
         if data is None:
-            if self.window.card.manga.tracking and self.window.card.manga.tracking.get(self.tracker.id):
+            tracked = self.window.card.manga.tracking and self.window.card.manga.tracking.get(self.tracker.id)
+
+            if access_token_valid and tracked:
                 thread = threading.Thread(target=run)
                 thread.daemon = True
                 thread.start()
             else:
                 self.set_expanded(False)
                 self.set_enable_expansion(False)
-                self.set_arrow_visible(False)
-                self.btn.set_visible(True)
                 self.action_row.set_title('')
+
+                if access_token_valid:
+                    self.set_arrow_visible(False)
+                    self.btn.set_visible(True)
+                    self.set_activatable(True)
+                    self.set_subtitle('')
+                else:
+                    self.set_arrow_visible(True)
+                    self.btn.set_visible(False)
+                    self.set_activatable(False)
+                    self.set_subtitle(_('Connection is expired'))
         else:
             complete(data)
 
-        return active
+        return granted and access_token_valid
 
     def on_btn_clicked(self, _btn):
         self.window.card.tracking_dialog.show_search(self.tracker)
@@ -478,7 +491,8 @@ class TrackingSearchSubPage(Adw.NavigationPage):
         def run(term):
             try:
                 results = self.tracker.search(term)
-            except Exception:
+            except Exception as e:
+                log_error_traceback(e)
                 results = None
 
             GLib.idle_add(complete, results)
