@@ -135,15 +135,9 @@ CREDITS = dict(
 
 RELEASE_NOTES = """
 <ul>
-    <li>[Library] Selection mode: Fixed read/unread actions</li>
-    <li>[Reader] Webtoon pager: Fixed high CPU usage</li>
-    <li>[Servers] Added Kavita</li>
-    <li>[Servers] Added MangaBall (23 languages)</li>
-    <li>[Servers] Anime-Sama (FR): Update</li>
-    <li>[Servers] Flame Comics (EN): Update</li>
-    <li>[Servers] MangaCrab (ES): Update</li>
-    <li>[Servers] Raijin Scan (FR): Update</li>
-    <li>[L10n] Updated Croatian, Czech and Russian translations</li>
+    <li>[Servers] Added Zenko (UK)</li>
+    <li>[Explorer] Servers: Kavita and Komga join Local in `Other` section</li>
+    <li>[L10n] Updated French, Russian and Ukrainian translations</li>
 </ul>
 <p>Happy reading.</p>
 """
