@@ -32,7 +32,7 @@ from gi.repository import WebKit
 from komikku.consts import REQUESTS_TIMEOUT
 from komikku.servers.exceptions import ChallengerError
 from komikku.servers.utils import get_session_cookies
-from komikku.utils import get_cache_dir
+from komikku.utils import get_webview_data_dir
 
 CF_RELOAD_MAX = 3
 DEBUG = False
@@ -125,14 +125,14 @@ class WebviewPage(Adw.NavigationPage):
 
         # Network session
         self.network_session = WebKit.NetworkSession.new(
-            os.path.join(get_cache_dir(), 'webview', 'data'),
-            os.path.join(get_cache_dir(), 'webview', 'cache')
+            os.path.join(get_webview_data_dir(), 'data'),
+            os.path.join(get_webview_data_dir(), 'cache')
         )
         self.network_session.get_website_data_manager().set_favicons_enabled(True)
         self.network_session.set_itp_enabled(False)
         self.network_session.get_cookie_manager().set_accept_policy(WebKit.CookieAcceptPolicy.ALWAYS)
         self.network_session.get_cookie_manager().set_persistent_storage(
-            os.path.join(get_cache_dir(), 'webview', 'cookies.sqlite'),
+            os.path.join(get_webview_data_dir(), 'cookies.sqlite'),
             WebKit.CookiePersistentStorage.SQLITE
         )
 
@@ -166,6 +166,9 @@ class WebviewPage(Adw.NavigationPage):
             self.exit()
             self.close_page()
 
+    def clear_data(self):
+        self.network_session.get_website_data_manager().clear(WebKit.WebsiteDataTypes.ALL, 0, None, None)
+
     def close_page(self, blank=True):
         self.disconnect_all_signals()
 
@@ -566,9 +569,6 @@ def eval_js(code):
 
         webview.connect_webview_signal('load-changed', on_load_changed)
 
-        if DEBUG:
-            webview.show()
-
     def on_evaluate_javascript_finish(_webkit_webview, result, _user_data=None):
         nonlocal error
         nonlocal res
@@ -618,9 +618,6 @@ def get_page_html(url, user_agent=None, wait_js_code=None, with_cookies=False):
         webview.connect_webview_signal('load-failed', on_load_failed)
         webview.connect_webview_signal('notify::title', on_title_changed)
 
-        if DEBUG:
-            webview.show()
-
     def on_get_cookies_finished(cookie_manager, result, _user_data):
         nonlocal cookies
 
@@ -657,7 +654,7 @@ def get_page_html(url, user_agent=None, wait_js_code=None, with_cookies=False):
             else:
                 webview.close_page()
         else:
-            error = f'Failed to get chapter page html: {url}'
+            error = f'Failed to get page html: {url}'
             webview.close_page()
 
     def on_load_changed(_webkit_webview, event):
@@ -673,8 +670,7 @@ def get_page_html(url, user_agent=None, wait_js_code=None, with_cookies=False):
     def on_load_failed(_webkit_webview, _event, _uri, _gerror):
         nonlocal error
 
-        error = f'Failed to load chapter page: {url}'
-
+        error = f'Failed to load page: {url}'
         webview.close_page()
 
     def on_title_changed(_webkit_webview, _title):
@@ -685,7 +681,7 @@ def get_page_html(url, user_agent=None, wait_js_code=None, with_cookies=False):
             webview.webkit_webview.evaluate_javascript('document.documentElement.outerHTML', -1, None, None, None, on_get_html_finish)
 
         elif webview.webkit_webview.props.title == 'abort':
-            error = f'Failed to get chapter page html: {url}'
+            error = f'Failed to get page html: {url}'
             webview.close_page()
 
     GLib.timeout_add(100, load_page)
@@ -700,6 +696,83 @@ def get_page_html(url, user_agent=None, wait_js_code=None, with_cookies=False):
     return html if not with_cookies else (html, cookies)
 
 
+def get_page_resources(url, paths, timeout=20, user_agent=None):
+    """
+    Returns all resources loaded by a page
+
+    :param url: Page URL
+    :param paths: List of paths to which resource URIs must match
+    :param timeout: Timeout in seconds (optional)
+    :param user_agent: User agent (optional)
+    """
+
+    data = None
+    error = None
+    resources = {}
+    ts_start = None
+    webview = Gio.Application.get_default().window.webview
+
+    def load_page():
+        nonlocal ts_start
+
+        if not webview.load_page(uri=url, user_agent=user_agent, auto_load_images=False):
+            return True
+
+        ts_start = time.time()
+
+        webview.connect_webview_signal('resource-load-started', on_resource_load_started)
+        webview.connect_webview_signal('load-changed', on_load_changed)
+        webview.connect_webview_signal('load-failed', on_load_failed)
+
+    def on_load_changed(_webkit_webview, event):
+        nonlocal data
+        if event == WebKit.LoadEvent.FINISHED:
+            data = resources
+
+    def on_resource_load_started(_webkit_webview, resource, request):
+        nonlocal resources
+
+        uri = request.get_uri()
+        if uri in resources:
+            return
+
+        found = False
+        for path in paths:
+            if path in uri:
+                found = True
+                break
+
+        if not found:
+            return
+
+        resources[uri] = dict(
+            uri=uri,
+            resource=resource,
+            request=request
+        )
+
+    def on_load_failed(_webkit_webview, _event, _uri, _gerror):
+        nonlocal error
+
+        error = f'Failed to load page: {url}'
+
+    GLib.timeout_add(100, load_page)
+
+    while (not ts_start or time.time() - ts_start < timeout) and data is None and error is None:
+        time.sleep(1)
+
+    if time.time() - ts_start > timeout:
+        error = f'Failed to load page (timeout): {url}'
+
+    webview.close_page()
+
+    if error:
+        logger.warning(error)
+        raise requests.exceptions.RequestException()
+
+    return list(data.values())
+
+
 def get_tracker_access_token(url, app_redirect_url, user_agent=None):
     """Use webview to request a client access token to a tracker
 
